@@ -39,6 +39,10 @@
  *
  */
 
+/**
+ * @file
+ */
+
 #include <sys/types.h>
 #include <sys/times.h>
 #include <sys/stat.h>
@@ -81,9 +85,17 @@ static char *_schema_name;
 static int (*_handle_scheduling)(void) = NULL;
 static int (*_handle_unused_argument)(const char *) = NULL;
 
+/**
+ * index values for additional command line options
+ */
 enum argv_short_options {
+  /*! --schema option */
   argv_option_schema = 256,
+
+  /*! --Xearlydebug option */
   argv_option_debug_early,
+
+  /*! --Xignoreunknown */
   argv_option_ignore_unknown,
 };
 
@@ -141,10 +153,15 @@ static const char *help_text =
 
 /**
  * Main program
+ * @param argc argument counter
+ * @param argv argument vector
+ * @param appdata application data
+ * @return application return code to shell
  */
 int
 oonf_main(int argc, char **argv, const struct oonf_appdata *appdata) {
   int return_code;
+  int result;
 
   /* early initialization */
   return_code = 1;
@@ -170,7 +187,7 @@ oonf_main(int argc, char **argv, const struct oonf_appdata *appdata) {
   }
 
   /* prepare plugin initialization */
-  oonf_plugins_init();
+  oonf_subsystem_init();
 
   /* initialize configuration system */
   if (oonf_cfg_init(argc, argv, appdata->default_cfg_handler)) {
@@ -191,8 +208,8 @@ oonf_main(int argc, char **argv, const struct oonf_appdata *appdata) {
   return_code = 1;
 
   /* read global section early */
-  if (oonf_cfg_update_globalcfg(true)) {
-    OONF_WARN(LOG_MAIN, "Cannot read global configuration section");
+  if ((result = oonf_cfg_update_globalcfg(true))) {
+    OONF_WARN(LOG_MAIN, "Cannot read global configuration section (%d)", result);
     goto oonf_cleanup;
   }
 
@@ -202,7 +219,7 @@ oonf_main(int argc, char **argv, const struct oonf_appdata *appdata) {
   }
 
   /* load plugins */
-  if (oonf_cfg_loadplugins()) {
+  if (oonf_cfg_load_subsystems()) {
     goto oonf_cleanup;
   }
 
@@ -221,7 +238,10 @@ oonf_main(int argc, char **argv, const struct oonf_appdata *appdata) {
     }
   }
 
-  if (config_global.lockfile != NULL && *config_global.lockfile != 0) {
+  if (appdata->need_lock
+      && config_global.lockfile != NULL
+      && *config_global.lockfile != 0
+      && strcmp(config_global.lockfile, "-") != 0) {
     /* create application lock */
     if (os_core_create_lockfile(config_global.lockfile)) {
       OONF_WARN(LOG_MAIN, "Could not acquire application lock '%s'",
@@ -272,15 +292,15 @@ oonf_main(int argc, char **argv, const struct oonf_appdata *appdata) {
   return_code = mainloop(argc, argv, appdata);
 
   /* tell plugins shutdown is in progress */
-  oonf_plugins_initiate_shutdown();
+  oonf_subsystem_initiate_shutdown();
 
   /* wait for 500 ms and process socket events */
   while(!_handle_scheduling());
 
 oonf_cleanup:
   /* free plugins */
-  oonf_cfg_unconfigure_plugins();
-  oonf_plugins_cleanup();
+  oonf_cfg_unconfigure_subsystems();
+  oonf_subsystem_cleanup();
 
   /* free logging/config bridge resources */
   oonf_logcfg_cleanup();
@@ -297,6 +317,11 @@ oonf_cleanup:
   return return_code;
 }
 
+/**
+ * Set the callback to the central scheduler
+ * @param scheduler pointer to scheduler function
+ * @return -1 if scheduler is already set
+ */
 int
 oonf_main_set_scheduler(int (*scheduler)(void)) {
   if (_handle_scheduling) {
@@ -307,6 +332,11 @@ oonf_main_set_scheduler(int (*scheduler)(void)) {
   return 0;
 }
 
+/**
+ * Set a handler for command line parameters that are not used by main()
+ * @param parameter_handler pointer to parameter handler function
+ * @return -1 if parameter handler is already set
+ */
 int
 oonf_main_set_parameter_handler(int (*parameter_handler)(const char *)) {
   if (_handle_unused_argument) {
@@ -316,6 +346,9 @@ oonf_main_set_parameter_handler(int (*parameter_handler)(const char *)) {
   return 0;
 }
 
+/**
+ * @return true if event scheduler should return to the mainloop
+ */
 bool
 oonf_main_shall_stop_scheduler(void) {
   return oonf_cfg_is_commit_set()
@@ -325,7 +358,7 @@ oonf_main_shall_stop_scheduler(void) {
 
 /**
  * Write process ID into file
- * @param filename
+ * @param filename name/path of pidfile
  * @return -1 if an error happened, 0 otherwise
  */
 static int
@@ -356,7 +389,7 @@ _write_pidfile(const char *filename) {
 
 /**
  * Handle incoming SIGINT signal
- * @param signo
+ * @param signo unused
  */
 static void
 quit_signal_handler(int signo __attribute__ ((unused))) {
@@ -365,7 +398,7 @@ quit_signal_handler(int signo __attribute__ ((unused))) {
 
 /**
  * Handle incoming SIGHUP signal
- * @param signo
+ * @param signo unused
  */
 static void
 hup_signal_handler(int signo __attribute__ ((unused))) {
@@ -374,6 +407,9 @@ hup_signal_handler(int signo __attribute__ ((unused))) {
 
 /**
  * Mainloop of olsrd
+ * @param argc argument counter
+ * @param argv argument vector
+ * @param appdata OONF appdata
  * @return exit code for olsrd
  */
 static int
@@ -468,8 +504,7 @@ parse_early_commandline(int argc, char **argv) {
  * Parse command line of olsrd
  * @param argc number of arguments
  * @param argv argument vector
- * @param def_config default configuration file, NULL if
- *   no default config file should be loaded
+ * @param appdata OONF appdata
  * @param reload_only true if only the command line arguments should
  *   be parsed that load a configuration (--set, --remove, --load,
  *   and --format), false for normal full parsing.
@@ -529,7 +564,7 @@ parse_commandline(int argc, char **argv,
         return_code = 0;
         break;
       case 'p':
-        if (oonf_cfg_load_plugin(optarg) == NULL) {
+        if (oonf_cfg_load_subsystem(optarg) == NULL) {
           return_code = 1;
         }
         else {
