@@ -1,8 +1,46 @@
+
 /*
- * dlep_base_radio.c
+ * The olsr.org Optimized Link-State Routing daemon version 2 (olsrd2)
+ * Copyright (c) 2004-2015, the olsr.org team - see HISTORY file
+ * All rights reserved.
  *
- *  Created on: Jun 29, 2015
- *      Author: rogge
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * * Redistributions of source code must retain the above copyright
+ *   notice, this list of conditions and the following disclaimer.
+ * * Redistributions in binary form must reproduce the above copyright
+ *   notice, this list of conditions and the following disclaimer in
+ *   the documentation and/or other materials provided with the
+ *   distribution.
+ * * Neither the name of olsr.org, olsrd nor the names of its
+ *   contributors may be used to endorse or promote products derived
+ *   from this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
+ * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
+ * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
+ * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
+ * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+ * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
+ * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
+ * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ * Visit http://www.olsr.org for more information.
+ *
+ * If you find this software useful feel free to make a donation
+ * to the project. For more information see the website or contact
+ * the copyright holders.
+ *
+ */
+
+/**
+ * @file
  */
 
 #include "common/common_types.h"
@@ -23,7 +61,7 @@
 static void _cb_init_router(struct dlep_session *);
 static void _cb_apply_router(struct dlep_session *);
 static void _cb_cleanup_router(struct dlep_session *);
-static void _cb_peer_create_discovery(void *);
+static void _cb_create_peer_discovery(struct oonf_timer_instance *);
 
 static int _router_process_peer_offer(struct dlep_extension *, struct dlep_session *);
 static int _router_process_peer_init_ack(struct dlep_extension *, struct dlep_session *);
@@ -108,11 +146,14 @@ static struct dlep_extension_implementation _router_signals[] = {
 
 static struct oonf_timer_class _peer_discovery_class = {
     .name = "dlep peer discovery",
-    .callback = _cb_peer_create_discovery,
+    .callback = _cb_create_peer_discovery,
     .periodic = true,
 };
 static struct dlep_extension *_base;
 
+/**
+ * Initialize the routers DLEP base protocol extension
+ */
 void
 dlep_base_proto_router_init(void) {
   _base = dlep_base_proto_init();
@@ -126,6 +167,10 @@ dlep_base_proto_router_init(void) {
   _base->cb_session_cleanup_router = _cb_cleanup_router;
 }
 
+/**
+ * Callback to initialize the router session
+ * @param session dlep session
+ */
 static void
 _cb_init_router(struct dlep_session *session) {
   if (session->restrict_signal == DLEP_PEER_INITIALIZATION_ACK) {
@@ -141,6 +186,10 @@ _cb_init_router(struct dlep_session *session) {
   }
 }
 
+/**
+ * Callback to apply new network settings to a router session
+ * @param session dlep session
+ */
 static void
 _cb_apply_router(struct dlep_session *session) {
   OONF_DEBUG(session->log_source, "Initialize base router session");
@@ -150,7 +199,6 @@ _cb_apply_router(struct dlep_session *session) {
      * so we need to send Peer Discovery messages
      */
     session->local_event_timer.class = &_peer_discovery_class;
-    session->local_event_timer.cb_context = session;
 
     OONF_DEBUG(session->log_source, "Activate discovery with interval %"
         PRIu64, session->cfg.discovery_interval);
@@ -161,21 +209,36 @@ _cb_apply_router(struct dlep_session *session) {
   }
 }
 
+/**
+ * Callback to cleanup the router session
+ * @param session dlep session
+ */
 static void
 _cb_cleanup_router(struct dlep_session *session) {
   struct oonf_layer2_net *l2net;
 
   l2net = oonf_layer2_net_get(session->l2_listener.name);
   if (l2net) {
+    /* remove DLEP mark from interface */
+    l2net->if_type = OONF_LAYER2_TYPE_UNDEFINED;
+    l2net->if_dlep = false;
+
+    /* and remove all DLEP data */
     oonf_layer2_net_remove(l2net, session->l2_origin);
   }
 
   dlep_base_proto_stop_timers(session);
 }
 
+/**
+ * Callback to generate regular peer discovery signals
+ * @param ptr timer instance that fired
+ */
 static void
-_cb_peer_create_discovery(void *ptr) {
-  struct dlep_session *session = ptr;
+_cb_create_peer_discovery(struct oonf_timer_instance *ptr) {
+  struct dlep_session *session;
+
+  session = container_of(ptr, struct dlep_session, local_event_timer);
 
   OONF_DEBUG(session->log_source, "Generate peer discovery");
 
@@ -186,6 +249,12 @@ _cb_peer_create_discovery(void *ptr) {
   session->cb_send_buffer(session, AF_INET6);
 }
 
+/**
+ * Process the peer offer signal
+ * @param ext (this) dlep extension
+ * @param session dlep session
+ * @return -1 if an error happened, 0 otherwise
+ */
 static int
 _router_process_peer_offer(
     struct dlep_extension *ext __attribute__((unused)),
@@ -193,10 +262,12 @@ _router_process_peer_offer(
   struct dlep_router_if *router_if;
   union netaddr_socket local, remote;
   struct dlep_parser_value *value;
+  const struct os_interface_ip *ip;
   const struct netaddr *result = NULL;
   struct netaddr addr;
   uint16_t port;
-  struct os_interface_data *ifdata;
+  bool tls;
+  struct os_interface *ifdata;
 
   if (session->restrict_signal != DLEP_PEER_OFFER) {
     /* ignore unless we are in discovery mode */
@@ -210,19 +281,24 @@ _router_process_peer_offer(
   result = NULL;
 
   /* remember interface data */
-  ifdata = &session->l2_listener.interface->data;
+  ifdata = session->l2_listener.data;
 
   /* IPv6 offer */
   value = dlep_session_get_tlv_value(session, DLEP_IPV6_CONPOINT_TLV);
   while (value) {
-    if (dlep_reader_ipv6_conpoint_tlv(&addr, &port, session, value)) {
+    if (dlep_reader_ipv6_conpoint_tlv(
+        &addr, &port, &tls, session, value)) {
       return -1;
     }
 
-    if (netaddr_is_in_subnet(&NETADDR_IPV6_LINKLOCAL, &addr)
+    if (tls) {
+      /* TLS not supported at the moment */
+    }
+    else if (netaddr_is_in_subnet(&NETADDR_IPV6_LINKLOCAL, &addr)
         || result == NULL) {
-      result = oonf_interface_get_prefix_from_dst(&addr, ifdata);
-      if (result) {
+      ip = os_interface_get_prefix_from_dst(&addr, ifdata);
+      if (ip) {
+        result = &ip->address;
         netaddr_socket_init(&remote, &addr, port, ifdata->index);
       }
     }
@@ -232,13 +308,19 @@ _router_process_peer_offer(
   /* IPv4 offer */
   value = dlep_session_get_tlv_value(session, DLEP_IPV4_CONPOINT_TLV);
   while (value && !result) {
-    if (dlep_reader_ipv4_conpoint_tlv(&addr, &port, session, value)) {
+    if (dlep_reader_ipv4_conpoint_tlv(&addr, &port, &tls, session, value)) {
       return -1;
     }
 
-    result = oonf_interface_get_prefix_from_dst(&addr, ifdata);
-    if (result) {
-      netaddr_socket_init(&remote, &addr, port, ifdata->index);
+    if (tls) {
+      /* TLS not supported at the moment */
+    }
+    else {
+      ip = os_interface_get_prefix_from_dst(&addr, ifdata);
+      if (ip) {
+        result = &ip->address;
+        netaddr_socket_init(&remote, &addr, port, ifdata->index);
+      }
     }
     value = dlep_session_get_next_tlv_value(session, value);
   }
@@ -246,13 +328,14 @@ _router_process_peer_offer(
   /* remote address of incoming session */
   if (!result) {
     netaddr_from_socket(&addr, &session->remote_socket);
-    result = oonf_interface_get_prefix_from_dst(&addr, ifdata);
-    if (!result) {
+    ip = os_interface_get_prefix_from_dst(&addr, ifdata);
+    if (!ip) {
       /* no possible way to communicate */
       OONF_DEBUG(session->log_source,
           "No matching prefix for incoming connection found");
       return -1;
     }
+    result = &ip->address;
     netaddr_socket_init(&remote, &addr, port, ifdata->index);
   }
 
@@ -268,6 +351,12 @@ _router_process_peer_offer(
   return -1;
 }
 
+/**
+ * Process the peer initialization ack message
+ * @param ext (this) dlep extension
+ * @param session dlep session
+ * @return -1 if an error happened, 0 otherwise
+ */
 static int
 _router_process_peer_init_ack(
     struct dlep_extension *ext __attribute__((unused)),
@@ -297,12 +386,20 @@ _router_process_peer_init_ack(
       return -1;
     }
   }
+  else if (dlep_session_update_extensions(session, NULL, 0)) {
+    return -1;
+  }
 
   l2net = oonf_layer2_net_add(session->l2_listener.name);
   if (!l2net) {
     return -1;
   }
 
+  /* mark interface as DLEP */
+  l2net->if_type = OONF_LAYER2_TYPE_WIRELESS;
+  l2net->if_dlep = true;
+
+  /* map user data into interface */
   result = dlep_reader_map_l2neigh_data(l2net->neighdata, session, _base);
   if (result) {
     OONF_INFO(session->log_source, "tlv mapping failed for extension %u: %u",
@@ -323,6 +420,12 @@ _router_process_peer_init_ack(
   return 0;
 }
 
+/**
+ * Process the peer update message
+ * @param ext (this) dlep extension
+ * @param session dlep session
+ * @return -1 if an error happened, 0 otherwise
+ */
 static int
 _router_process_peer_update(
     struct dlep_extension *ext __attribute__((unused)),
@@ -342,10 +445,19 @@ _router_process_peer_update(
     return -1;
   }
 
-  // we don't support IP address exchange at the moment
-  return 0;
+  /* we don't support IP address exchange at the moment */
+
+  /* generate ACK */
+  return dlep_session_generate_signal(
+      session, DLEP_PEER_UPDATE_ACK, NULL);
 }
 
+/**
+ * Process the peer update ack message
+ * @param ext (this) dlep extension
+ * @param session dlep session
+ * @return -1 if an error happened, 0 otherwise
+ */
 static int
 _router_process_peer_update_ack(
     struct dlep_extension *ext __attribute__((unused)),
@@ -354,6 +466,12 @@ _router_process_peer_update_ack(
   return 0;
 }
 
+/**
+ * Process the destination up message
+ * @param ext (this) dlep extension
+ * @param session dlep session
+ * @return -1 if an error happened, 0 otherwise
+ */
 static int
 _router_process_destination_up(
     struct dlep_extension *ext __attribute__((unused)),
@@ -388,10 +506,17 @@ _router_process_destination_up(
     return result;
   }
 
+  /* generate ACK */
   return dlep_session_generate_signal(
       session, DLEP_DESTINATION_UP_ACK, &mac);
 }
 
+/**
+ * Process the destination up ack message
+ * @param ext (this) dlep extension
+ * @param session dlep session
+ * @return -1 if an error happened, 0 otherwise
+ */
 static int
 _router_process_destination_up_ack(
     struct dlep_extension *ext __attribute__((unused)),
@@ -400,6 +525,12 @@ _router_process_destination_up_ack(
   return 0;
 }
 
+/**
+ * Process the destination down message
+ * @param ext (this) dlep extension
+ * @param session dlep session
+ * @return -1 if an error happened, 0 otherwise
+ */
 static int
 _router_process_destination_down(
     struct dlep_extension *ext __attribute__((unused)),
@@ -425,10 +556,17 @@ _router_process_destination_down(
   /* remove layer2 neighbor */
   oonf_layer2_neigh_remove(l2neigh, session->l2_origin);
 
+  /* generate ACK */
   return dlep_session_generate_signal(
-      session, DLEP_DESTINATION_UP_ACK, &mac);
+      session, DLEP_DESTINATION_DOWN_ACK, &mac);
 }
 
+/**
+ * Process the destination down ack message
+ * @param ext (this) dlep extension
+ * @param session dlep session
+ * @return -1 if an error happened, 0 otherwise
+ */
 static int
 _router_process_destination_down_ack(
     struct dlep_extension *ext __attribute__((unused)),
@@ -437,6 +575,12 @@ _router_process_destination_down_ack(
   return 0;
 }
 
+/**
+ * Process the destination update message
+ * @param ext (this) dlep extension
+ * @param session dlep session
+ * @return -1 if an error happened, 0 otherwise
+ */
 static int
 _router_process_destination_update(
     struct dlep_extension *ext __attribute__((unused)),
@@ -471,6 +615,12 @@ _router_process_destination_update(
   return 0;
 }
 
+/**
+ * Process the link characteristic ack message
+ * @param ext (this) dlep extension
+ * @param session dlep session
+ * @return -1 if an error happened, 0 otherwise
+ */
 static int
 _router_process_link_char_ack(
     struct dlep_extension *ext __attribute__((unused)),
@@ -479,6 +629,13 @@ _router_process_link_char_ack(
   return 0;
 }
 
+/**
+ * Generate a peer discovery signal
+ * @param ext (this) dlep extension
+ * @param session dlep session
+ * @param addr mac address the message should refer to
+ * @return -1 if an error happened, 0 otherwise
+ */
 static int
 _router_write_peer_discovery(
     struct dlep_extension *ext __attribute__((unused)),
@@ -490,6 +647,13 @@ _router_write_peer_discovery(
   return 0;
 }
 
+/**
+ * Generate a peer init message
+ * @param ext (this) dlep extension
+ * @param session dlep session
+ * @param addr mac address the message should refer to
+ * @return -1 if an error happened, 0 otherwise
+ */
 static int
 _router_write_peer_init(
     struct dlep_extension *ext __attribute__((unused)),
