@@ -75,16 +75,12 @@ static struct oonf_class _router_session_class = {
   .size = sizeof(struct dlep_router_session),
 };
 
-static uint32_t _l2_origin;
-
 /**
  * Initialize dlep router session framework
  */
 void
 dlep_router_session_init(void) {
   oonf_class_add(&_router_session_class);
-
-  _l2_origin = oonf_layer2_register_origin();
 }
 
 /**
@@ -137,8 +133,8 @@ dlep_router_add_session(struct dlep_router_if *interf,
   }
 
   /* initialize tree node */
-  memcpy(&router_session->remote_socket, remote, sizeof(*remote));
-  router_session->_node.key = &router_session->remote_socket;
+  memcpy(&router_session->session.remote_socket, remote, sizeof(*remote));
+  router_session->_node.key = &router_session->session.remote_socket;
 
   /* configure and open TCP session */
   router_session->tcp.config.session_timeout = 120000; /* 120 seconds */
@@ -159,10 +155,7 @@ dlep_router_add_session(struct dlep_router_if *interf,
     return NULL;
   }
 
-  /* copy socket information */
-  memcpy(&router_session->session.remote_socket, remote,
-      sizeof(router_session->session.remote_socket));
-
+  /* open stream */
   router_session->stream = oonf_stream_connect_to(&router_session->tcp, remote);
   if (!router_session->stream) {
     OONF_WARN(LOG_DLEP_ROUTER,
@@ -184,8 +177,6 @@ dlep_router_add_session(struct dlep_router_if *interf,
   router_session->session.cb_end_session = _cb_end_session;
   memcpy(&router_session->session.cfg, &interf->interf.session.cfg,
       sizeof(router_session->session.cfg));
-  memcpy(&router_session->session.remote_socket, remote,
-      sizeof(router_session->session.remote_socket));
 
   /* initialize back pointer */
   router_session->interface = interf;
@@ -250,8 +241,8 @@ _cb_tcp_lost(struct oonf_stream_session *tcp_session) {
 
 /**
  * Receive tcp data via oonf_stream_socket
- * @param tcp_session
- * @return
+ * @param tcp_session tcp session to DLEP partner
+ * @return new state of TCP session
  */
 static enum oonf_stream_session_state
 _cb_tcp_receive_data(struct oonf_stream_session *tcp_session) {
@@ -288,7 +279,7 @@ _cb_send_buffer(struct dlep_session *session,
 
 /**
  * Callback triggered when session is terminated
- * @param session
+ * @param session dlep session
  */
 static void
 _cb_end_session(struct dlep_session *session) {
