@@ -39,6 +39,10 @@
  *
  */
 
+/**
+ * @file
+ */
+
 #include "common/avl.h"
 #include "common/common_types.h"
 #include "common/list.h"
@@ -55,14 +59,28 @@
 #include "olsrv2/olsrv2_internal.h"
 #include "olsrv2/olsrv2_lan.h"
 #include "olsrv2/olsrv2_originator.h"
+#include "olsrv2/olsrv2_routing.h"
 #include "olsrv2/olsrv2_writer.h"
 
 /* constants */
+
+/**
+ * olsrv2 index values for address tlvs
+ */
 enum olsrv2_addrtlv_idx {
+  /*! index of neighbor address tlv */
   IDX_ADDRTLV_NBR_ADDR_TYPE,
+
+  /*! index of destination specific gateway tlv */
   IDX_ADDRTLV_GATEWAY_DSTSPEC,
+
+  /*! index of source specific gateway tlv */
   IDX_ADDRTLV_GATEWAY_SRCSPEC,
+
+  /*! index of source specific default gateway tlv */
   IDX_ADDRTLV_GATEWAY_SRCSPEC_DEF,
+
+  /*! index of source prefix tlv */
   IDX_ADDRTLV_GATEWAY_SRC_PREFIX,
 };
 
@@ -88,7 +106,7 @@ static void _cb_finishMessageTLVs(struct rfc5444_writer *,
 static struct rfc5444_writer_message *_olsrv2_message = NULL;
 
 static struct rfc5444_writer_content_provider _olsrv2_msgcontent_provider = {
-  .msg_type = RFC5444_MSGTYPE_TC,
+  .msg_type = RFC7181_MSGTYPE_TC,
   .addMessageTLVs = _cb_addMessageTLVs,
   .addAddresses = _cb_addAddresses,
   .finishMessageTLVs = _cb_finishMessageTLVs,
@@ -121,7 +139,7 @@ olsrv2_writer_init(struct oonf_rfc5444_protocol *protocol) {
   _protocol = protocol;
 
   _olsrv2_message = rfc5444_writer_register_message(
-      &_protocol->writer, RFC5444_MSGTYPE_TC, false);
+      &_protocol->writer, RFC7181_MSGTYPE_TC, false);
   if (_olsrv2_message == NULL) {
     OONF_WARN(LOG_OLSRV2, "Could not register OLSRV2 TC message");
     return -1;
@@ -179,8 +197,7 @@ olsrv2_writer_send_tc(void) {
 void
 olsrv2_writer_set_forwarding_selector(
     bool (*forward_target_selector)(struct rfc5444_writer_target *,
-      struct rfc5444_reader_tlvblock_context *context,
-      const uint8_t *buffer, size_t len)) {
+      struct rfc5444_reader_tlvblock_context *context)) {
   if (forward_target_selector) {
     _olsrv2_message->forward_target_selector = forward_target_selector;
   }
@@ -200,15 +217,15 @@ _send_tc(int af_type) {
   originator = olsrv2_originator_get(af_type);
   if (netaddr_get_address_family(originator) == af_type) {
     OONF_INFO(LOG_OLSRV2_W, "Emit IPv%d TC message.", af_type == AF_INET ? 4 : 6);
-    oonf_rfc5444_send_all(_protocol, RFC5444_MSGTYPE_TC,
+    oonf_rfc5444_send_all(_protocol, RFC7181_MSGTYPE_TC,
         af_type == AF_INET ? 4 : 16, nhdp_flooding_selector);
   }
 }
 
 /**
  * Callback for rfc5444 writer to add message header for tc
- * @param writer
- * @param message
+ * @param writer RFC5444 writer instance
+ * @param message RFC5444 message that is generated
  */
 static int
 _cb_addMessageHeader(struct rfc5444_writer *writer,
@@ -247,7 +264,7 @@ _cb_finishMessageHeader(struct rfc5444_writer *writer,
 
 /**
  * Callback for rfc5444 writer to add message tlvs to tc
- * @param writer
+ * @param writer RFC5444 writer instance
  */
 static void
 _cb_addMessageTLVs(struct rfc5444_writer *writer) {
@@ -313,7 +330,7 @@ _generate_neighbor_metric_tlvs(struct rfc5444_writer *writer,
 
     metric_in = neigh_domain->metric.in;
     if (metric_in > RFC7181_METRIC_MAX) {
-      /*  */
+      /* Metric value does not make sense */
       continue;
     }
     if (rfc7181_metric_encode(&metric_in_encoded, metric_in)) {
@@ -355,11 +372,10 @@ _generate_neighbor_metric_tlvs(struct rfc5444_writer *writer,
 
 /**
  * Callback for rfc5444 writer to add addresses and addresstlvs to tc
- * @param writer
+ * @param writer RFC5444 writer instance
  */
 static void
 _cb_addAddresses(struct rfc5444_writer *writer) {
-  const struct netaddr_acl *routable_acl;
   struct rfc5444_writer_address *addr;
   struct nhdp_neighbor *neigh;
   struct nhdp_naddr *naddr;
@@ -378,7 +394,6 @@ _cb_addAddresses(struct rfc5444_writer *writer) {
   struct netaddr_str nbuf1, nbuf2;
 #endif
 
-  routable_acl = olsrv2_get_routable();
   af_type = writer->msg_addr_len == 4 ? AF_INET : AF_INET6;
 
   /* iterate over neighbors */
@@ -411,9 +426,15 @@ _cb_addAddresses(struct rfc5444_writer *writer) {
         continue;
       }
 
+      if (!olsrv2_is_nhdp_routable(&naddr->neigh_addr)
+          && netaddr_cmp(&neigh->originator, &naddr->neigh_addr) != 0) {
+        /* do not propagate unroutable addresses in TCs */
+        continue;
+      }
+
       nbr_addrtype_value = 0;
 
-      if (netaddr_acl_check_accept(routable_acl, &naddr->neigh_addr)) {
+      if (olsrv2_is_routable(&naddr->neigh_addr)) {
         nbr_addrtype_value |= RFC7181_NBR_ADDR_TYPE_ROUTABLE;
       }
       if (netaddr_cmp(&neigh->originator, &naddr->neigh_addr) == 0) {
@@ -485,7 +506,8 @@ _cb_addAddresses(struct rfc5444_writer *writer) {
     list_for_each_element(nhdp_domain_get_list(), domain, _node) {
       lan_data = olsrv2_lan_get_domaindata(domain, lan);
       metric_out = lan_data->outgoing_metric;
-      if (metric_out >= RFC7181_METRIC_INFINITE) {
+      if (metric_out > RFC7181_METRIC_MAX) {
+        /* metric value does not make sense */
         continue;
       }
 
@@ -531,10 +553,10 @@ _cb_addAddresses(struct rfc5444_writer *writer) {
 
 /**
  * Callback triggered when tc is finished.
- * @param writer
- * @param start
- * @param end
- * @param complete
+ * @param writer RFC5444 writer instance
+ * @param start first address contained in generated message
+ * @param end last address contained in generated message
+ * @param complete true if all addresses are in message, false otherwise
  */
 static void
 _cb_finishMessageTLVs(struct rfc5444_writer *writer,
@@ -544,7 +566,7 @@ _cb_finishMessageTLVs(struct rfc5444_writer *writer,
   uint16_t ansn;
 
   /* get ANSN */
-  ansn = htons(olsrv2_update_ansn());
+  ansn = htons(olsrv2_routing_get_ansn());
 
   rfc5444_writer_set_messagetlv(writer, RFC7181_MSGTLV_CONT_SEQ_NUM,
       complete ? RFC7181_CONT_SEQ_NUM_COMPLETE : RFC7181_CONT_SEQ_NUM_INCOMPLETE,
