@@ -74,11 +74,16 @@ struct _rfc5444_config {
 
   /*! IP protocol number to be used for RFC5444 communication */
   int ip_proto;
+};
 
-  /**
-   * interval to wait for aggregating
-   * RFC5444 messages on the same target
-   */
+/**
+ * RFC5444 interface specific configuration
+ */
+struct _rfc5444_if_config {
+  /*! packet socket configuration */
+  struct oonf_packet_managed_config sock;
+
+  /*! maximum aggregation interval for this interface */
   uint64_t aggregation_interval;
 };
 
@@ -89,6 +94,9 @@ static void _cleanup(void);
 static struct oonf_rfc5444_target *_create_target(
     struct oonf_rfc5444_interface *, struct netaddr *dst, bool unicast);
 static void _destroy_target(struct oonf_rfc5444_target *);
+static void _print_packet_to_buffer(enum oonf_log_source source,
+    union netaddr_socket *sock, struct oonf_rfc5444_interface *interf,
+    const uint8_t *ptr, size_t len, const char *success, const char *error);
 
 static void _cb_receive_data(struct oonf_packet_socket *,
       union netaddr_socket *from, void *ptr, size_t length);
@@ -97,8 +105,8 @@ static void _cb_send_unicast_packet(
 static void _cb_send_multicast_packet(
     struct rfc5444_writer *, struct rfc5444_writer_target *, void *, size_t);
 static void _cb_forward_message(struct rfc5444_reader_tlvblock_context *context,
-    uint8_t *buffer, size_t length);
-static void _cb_forwarding_notifier(struct rfc5444_writer_target *);
+    const uint8_t *buffer, size_t length);
+static void _cb_msggen_notifier(struct rfc5444_writer_target *);
 
 static bool _cb_single_target_selector(struct rfc5444_writer *, struct rfc5444_writer_target *, void *);
 static bool _cb_filtered_targets_selector(struct rfc5444_writer *writer,
@@ -172,8 +180,6 @@ static struct cfg_schema_entry _rfc5444_entries[] = {
     "UDP port for RFC5444 interface", 0, false, 1, 65535),
   CFG_MAP_INT32_MINMAX(_rfc5444_config, ip_proto, "ip_proto", RFC5444_MANET_IPPROTO_TXT,
     "IP protocol for RFC5444 interface", 0, false, 1, 255),
-  CFG_MAP_CLOCK(_rfc5444_config, aggregation_interval, "agregation_interval", "0.100",
-    "Interval in seconds for message aggregation"),
 };
 
 static struct cfg_schema_section _rfc5444_section = {
@@ -185,19 +191,24 @@ static struct cfg_schema_section _rfc5444_section = {
 };
 
 static struct cfg_schema_entry _interface_entries[] = {
-  CFG_MAP_ACL_V46(oonf_packet_managed_config, acl, "acl", ACL_DEFAULT_ACCEPT,
+  CFG_MAP_ACL_V46(_rfc5444_if_config, sock.acl, "acl", ACL_DEFAULT_ACCEPT,
     "Access control list for RFC5444 interface"),
-  CFG_MAP_ACL_V46(oonf_packet_managed_config, bindto, "bindto",
+  CFG_MAP_ACL_V46(_rfc5444_if_config, sock.bindto, "bindto",
       "-127.0.0.0/8\0" "fe80::/10\0" "-::/0\0" ACL_FIRST_ACCEPT "\0" ACL_DEFAULT_ACCEPT,
     "Bind RFC5444 socket to an address matching this filter (both IPv4 and IPv6)"),
-  CFG_MAP_NETADDR_V4(oonf_packet_managed_config, multicast_v4, "multicast_v4", RFC5444_MANET_MULTICAST_V4_TXT,
+  CFG_MAP_NETADDR_V4(_rfc5444_if_config, sock.multicast_v4, "multicast_v4", RFC5444_MANET_MULTICAST_V4_TXT,
     "ipv4 multicast address of this socket", false, true),
-  CFG_MAP_NETADDR_V6(oonf_packet_managed_config, multicast_v6, "multicast_v6", RFC5444_MANET_MULTICAST_V6_TXT,
+  CFG_MAP_NETADDR_V6(_rfc5444_if_config, sock.multicast_v6, "multicast_v6", RFC5444_MANET_MULTICAST_V6_TXT,
     "ipv6 multicast address of this socket", false, true),
-  CFG_MAP_INT32_MINMAX(oonf_packet_managed_config, dscp, "dscp", "192",
+  CFG_MAP_INT32_MINMAX(_rfc5444_if_config, sock.dscp, "dscp", "192",
     "DSCP field for outgoing UDP protocol traffic", 0, false, 0, 255),
-  CFG_MAP_BOOL(oonf_packet_managed_config, rawip, "rawip", "false",
+  CFG_MAP_BOOL(_rfc5444_if_config, sock.rawip, "rawip", "false",
     "True if a raw IP socket should be used, false to use UDP"),
+  CFG_MAP_INT32_MINMAX(_rfc5444_if_config, sock.ttl_multicast, "multicast_ttl", "1",
+    "TTL value of outgoing multicast traffic", 0, false, 1, 255),
+  CFG_MAP_CLOCK(_rfc5444_if_config, aggregation_interval, "aggregation_interval", "0.100",
+    "Interval in seconds for message aggregation"),
+
 };
 
 static struct cfg_schema_section _interface_section = {
@@ -209,8 +220,6 @@ static struct cfg_schema_section _interface_section = {
   .next_section = &_rfc5444_section,
 };
 
-static uint64_t _aggregation_interval;
-
 /* rfc5444 handling */
 static const struct rfc5444_reader _reader_template = {
   .forward_message = _cb_forward_message,
@@ -394,11 +403,6 @@ enum rfc5444_result oonf_rfc5444_send_if(
     return RFC5444_OKAY;
   }
 
-  if (!oonf_timer_is_active(&target->_aggregation)) {
-    /* activate aggregation timer */
-    oonf_timer_start(&target->_aggregation, _aggregation_interval);
-  }
-
   /* create message */
   OONF_INFO(LOG_RFC5444, "Create message id %d for protocol %s/target %s on interface %s",
       msgid, target->interface->protocol->name, netaddr_to_string(&buf, &target->dst),
@@ -461,7 +465,7 @@ oonf_rfc5444_add_protocol(const char *name, bool fixed_local_port) {
     rfc5444_reader_init(&protocol->reader);
     rfc5444_writer_init(&protocol->writer);
 
-    protocol->writer.forwarding_notifier = _cb_forwarding_notifier;
+    protocol->writer.message_generation_notifier = _cb_msggen_notifier;
 
     /* initialize processing and forwarding set */
     oonf_duplicate_set_add(&protocol->forwarded_set, OONF_DUPSET_16BIT);
@@ -732,8 +736,6 @@ oonf_rfc5444_reconfigure_interface(struct oonf_rfc5444_interface *interf,
       interf->name, interf->_socket_config.port, interf->_socket_config.multicast_port,
       interf->_socket_config.protocol);
 
-  OONF_DEBUG(LOG_RFC5444, "compare: '%s' == '%s'",
-      interf->name, RFC5444_UNICAST_INTERFACE);
   if (strcmp(interf->name, RFC5444_UNICAST_INTERFACE) == 0) {
     /* unicast interface */
     netaddr_invalidate(&interf->_socket_config.multicast_v4);
@@ -866,6 +868,76 @@ oonf_rfc5444_remove_target(struct oonf_rfc5444_target *target) {
   _destroy_target(target);
 }
 
+/**
+ * Send a raw RFC5444 packet to a target
+ * @param target target for the packet data
+ * @param ptr pointer to data
+ * @param len length of data
+ */
+void
+oonf_rfc5444_send_target_data(struct oonf_rfc5444_target *target,
+    const void *ptr, size_t len) {
+  union netaddr_socket sock;
+  struct os_interface_listener *interf;
+
+  interf = oonf_rfc5444_get_core_if_listener(target->interface);
+  netaddr_socket_init(&sock, &target->dst, target->interface->protocol->port,
+      interf->data->index);
+
+  _print_packet_to_buffer(LOG_RFC5444_W, &sock, target->interface, ptr, len,
+      "Outgoing RFC5444 packet to",
+      "Error while parsing outgoing RFC5444 packet to");
+
+  if (_block_output) {
+    OONF_DEBUG(LOG_RFC5444, "Output blocked");
+    return;
+  }
+  if (target == target->interface->multicast4
+      || target == target->interface->multicast6) {
+    oonf_packet_send_managed_multicast(&target->interface->_socket,
+        ptr, len, netaddr_get_address_family(&target->dst));
+  }
+  else {
+    oonf_packet_send_managed(&target->interface->_socket, &sock, ptr, len);
+  }
+}
+
+/**
+ * Send a raw RFC5444 packet through an interface to a destination address
+ * @param interf rfc5444 interface
+ * @param dst destination address for packet
+ * @param ptr pointer to data
+ * @param len length of data
+ */
+void
+oonf_rfc5444_send_interface_data(struct oonf_rfc5444_interface *interf,
+    const struct netaddr *dst, const void *ptr, size_t len) {
+  union netaddr_socket sock;
+  struct os_interface_listener *os_interf;
+
+  os_interf = oonf_rfc5444_get_core_if_listener(interf);
+  netaddr_socket_init(&sock, dst, interf->protocol->port,
+      os_interf->data->index);
+
+  _print_packet_to_buffer(LOG_RFC5444_W, &sock, interf, ptr, len,
+      "Outgoing RFC5444 packet to",
+      "Error while parsing outgoing RFC5444 packet to");
+
+  if (_block_output) {
+    OONF_DEBUG(LOG_RFC5444, "Output blocked");
+    return;
+  }
+
+  if (netaddr_is_in_subnet(&NETADDR_IPV4_MULTICAST, dst)
+      || netaddr_is_in_subnet(&NETADDR_IPV6_MULTICAST, dst)) {
+    oonf_packet_send_managed_multicast(&interf->_socket,
+        ptr, len, netaddr_get_address_family(dst));
+  }
+  else {
+    oonf_packet_send_managed(&interf->_socket, &sock, ptr, len);
+  }
+}
+
 /**
  * @param target oonf rfc5444 target
  * @return local socket corresponding to target destination
@@ -988,7 +1060,7 @@ static void
 _print_packet_to_buffer(enum oonf_log_source source,
     union netaddr_socket *sock __attribute__((unused)),
     struct oonf_rfc5444_interface *interf __attribute__((unused)),
-    uint8_t *ptr, size_t len,
+    const uint8_t *ptr, size_t len,
     const char *success __attribute__((unused)),
     const char *error __attribute__((unused))) {
   enum rfc5444_result result;
@@ -1037,11 +1109,11 @@ _cb_receive_data(struct oonf_packet_socket *sock,
     return;
   }
 
-  protocol->input_socket = from;
-  protocol->input_address = &source_ip;
+  protocol->input.src_socket = from;
+  protocol->input.src_address = &source_ip;
+  protocol->input.interface = interf;
 
-  protocol->input_interface = interf;
-  protocol->input_is_multicast =
+  protocol->input.is_multicast =
       sock == &interf->_socket.multicast_v4
       || sock == &interf->_socket.multicast_v6;
 
@@ -1072,9 +1144,9 @@ _cb_receive_data(struct oonf_packet_socket *sock,
 /**
  * Callback for sending a multicast packet to a rfc5444 target
  * @param writer rfc5444 writer
- * @param interf rfc5444 interface
+ * @param target rfc5444 target
  * @param ptr pointer to outgoing buffer
- * @param size_t length of buffer
+ * @param len length of buffer
  */
 static void
 _cb_send_multicast_packet(struct rfc5444_writer *writer __attribute__((unused)),
@@ -1104,9 +1176,9 @@ _cb_send_multicast_packet(struct rfc5444_writer *writer __attribute__((unused)),
 /**
  * Callback for sending an unicast packet to a rfc5444 target
  * @param writer rfc5444 writer
- * @param interf rfc5444 interface
+ * @param target rfc5444 target
  * @param ptr pointer to outgoing buffer
- * @param size_t length of buffer
+ * @param len length of buffer
  */
 static void
 _cb_send_unicast_packet(struct rfc5444_writer *writer __attribute__((unused)),
@@ -1135,14 +1207,14 @@ _cb_send_unicast_packet(struct rfc5444_writer *writer __attribute__((unused)),
 
 /**
  * Handle forwarding of rfc5444 messages
- * @param context
- * @param buffer
- * @param length
+ * @param context RFC5444 tlvblock reader context
+ * @param buffer message to be forwarded
+ * @param length length of message
  */
 static void
 _cb_forward_message(
     struct rfc5444_reader_tlvblock_context *context,
-    uint8_t *buffer, size_t length) {
+    const uint8_t *buffer, size_t length) {
   struct oonf_rfc5444_protocol *protocol;
   enum rfc5444_result result;
 
@@ -1160,13 +1232,13 @@ _cb_forward_message(
 }
 
 static void
-_cb_forwarding_notifier(struct rfc5444_writer_target *rfc5444target) {
+_cb_msggen_notifier(struct rfc5444_writer_target *rfc5444target) {
   struct oonf_rfc5444_target *target;
 
   target = container_of(rfc5444target, struct oonf_rfc5444_target, rfc5444_target);
   if (!oonf_timer_is_active(&target->_aggregation)) {
     /* activate aggregation timer */
-    oonf_timer_start(&target->_aggregation, _aggregation_interval);
+    oonf_timer_start(&target->_aggregation, target->interface->aggregation_interval);
   }
 }
 
@@ -1188,7 +1260,7 @@ _cb_single_target_selector(struct rfc5444_writer *writer __attribute__((unused))
 /**
  * Selector for outgoing target
  * @param writer rfc5444 writer
- * @param target rfc5444 target
+ * @param rfc5444_target rfc5444 target
  * @param ptr custom pointer, contains rfc5444 target
  * @return true if target corresponds to selection
  */
@@ -1214,11 +1286,6 @@ _cb_filtered_targets_selector(struct rfc5444_writer *writer,
     return false;
   }
 
-  if (!oonf_timer_is_active(&target->_aggregation)) {
-    /* activate aggregation timer */
-    oonf_timer_start(&target->_aggregation, _aggregation_interval);
-  }
-
   /* create message */
   OONF_INFO(LOG_RFC5444, "Send message to protocol %s/target %s on interface %s",
       target->interface->protocol->name, netaddr_to_string(&buf, &target->dst),
@@ -1265,7 +1332,7 @@ _alloc_addrtlv_entry(void) {
 
 /**
  * Free an addrblock entry
- * @param pointer to addrblock
+ * @param addrblock addressblock to be freed
  */
 static void
 _free_addrblock_entry(struct rfc5444_reader_addrblock_entry* addrblock) {
@@ -1274,7 +1341,7 @@ _free_addrblock_entry(struct rfc5444_reader_addrblock_entry* addrblock) {
 
 /**
  * Free a tlvblock entry
- * @param pointer to tlvblock
+ * @param tlvblock tlvblock to be freed
  */
 static void
 _free_tlvblock_entry(struct rfc5444_reader_tlvblock_entry *tlvblock) {
@@ -1282,8 +1349,8 @@ _free_tlvblock_entry(struct rfc5444_reader_tlvblock_entry *tlvblock) {
 }
 
 /**
- * Free a tlvblock entry
- * @param pointer to tlvblock
+ * Free an address
+ * @param address address to be freed
  */
 static void
 _free_address_entry(struct rfc5444_writer_address *address) {
@@ -1291,8 +1358,8 @@ _free_address_entry(struct rfc5444_writer_address *address) {
 }
 
 /**
- * Free a tlvblock entry
- * @param pointer to tlvblock
+ * Free an address tlv entry
+ * @param addrtlv address tlv entry to be freed
  */
 static void
 _free_addrtlv_entry(struct rfc5444_writer_addrtlv *addrtlv) {
@@ -1302,7 +1369,7 @@ _free_addrtlv_entry(struct rfc5444_writer_addrtlv *addrtlv) {
 /**
  * Callback to add sequence number to outgoing RFC5444 packet
  * @param writer pointer to rfc5444 writer
- * @param interf pointer to rfc5444 interface
+ * @param rfc5444_target rfc5444 target where the packet will be sent to
  */
 static void
 _cb_add_seqno(struct rfc5444_writer *writer, struct rfc5444_writer_target *rfc5444_target) {
@@ -1356,7 +1423,6 @@ _cb_cfg_rfc5444_changed(void) {
   /* apply values */
   oonf_rfc5444_reconfigure_protocol(_rfc5444_protocol,
       config.port, config.ip_proto);
-  _aggregation_interval = config.aggregation_interval;
 }
 
 /**
@@ -1364,7 +1430,7 @@ _cb_cfg_rfc5444_changed(void) {
  */
 static void
 _cb_cfg_interface_changed(void) {
-  struct oonf_packet_managed_config config;
+  struct _rfc5444_if_config config;
   struct oonf_rfc5444_interface *interf;
   const char *ifname;
   char ifbuf[IF_NAMESIZE];
@@ -1403,16 +1469,17 @@ _cb_cfg_interface_changed(void) {
     }
   }
 
-  oonf_rfc5444_reconfigure_interface(interf, &config);
+  oonf_rfc5444_reconfigure_interface(interf, &config.sock);
+  interf->aggregation_interval = config.aggregation_interval;
 
   /* fall through */
 interface_changed_cleanup:
-  oonf_packet_free_managed_config(&config);
+  oonf_packet_free_managed_config(&config.sock);
 }
 
 /**
  * Interface settings of a rfc5444 interface changed
- * @param managed
+ * @param managed managed socket which interface changed
  * @param changed true if socket addresses changed
  */
 static void
