@@ -53,9 +53,9 @@
 #include "core/oonf_logging.h"
 #include "core/oonf_subsystem.h"
 #include "subsystems/oonf_class.h"
-#include "subsystems/oonf_interface.h"
 #include "subsystems/oonf_timer.h"
 #include "subsystems/oonf_stream_socket.h"
+#include "subsystems/os_interface.h"
 #include "subsystems/os_system.h"
 #include "subsystems/os_fd.h"
 
@@ -69,7 +69,7 @@ static void _cleanup(void);
 static void _stream_close(struct oonf_stream_session *session);
 int _apply_managed(struct oonf_stream_managed *managed);
 static int _apply_managed_socket(int af_type, struct oonf_stream_managed *managed,
-    struct oonf_stream_socket *stream, struct os_interface_data *data);
+    struct oonf_stream_socket *stream, struct os_interface *os_if);
 static void _cb_parse_request(struct oonf_socket_entry *);
 static struct oonf_stream_session *_create_session(
     struct oonf_stream_socket *stream_socket, struct os_fd *sock,
@@ -78,7 +78,7 @@ static struct oonf_stream_session *_create_session(
 static void _cb_parse_connection(struct oonf_socket_entry *entry);
 
 static void _cb_timeout_handler(struct oonf_timer_instance *);
-static int _cb_interface_listener(struct oonf_interface_listener *l);
+static int _cb_interface_listener(struct os_interface_listener *listener);
 
 /* list of olsr stream sockets */
 static struct list_entity _stream_head;
@@ -97,11 +97,11 @@ static struct oonf_timer_class _connection_timeout = {
 /* subsystem definition */
 static const char *_dependencies[] = {
   OONF_CLASS_SUBSYSTEM,
-  OONF_INTERFACE_SUBSYSTEM,
   OONF_SOCKET_SUBSYSTEM,
   OONF_TIMER_SUBSYSTEM,
-  OONF_OS_SYSTEM_SUBSYSTEM,
   OONF_OS_FD_SUBSYSTEM,
+  OONF_OS_INTERFACE_SUBSYSTEM,
+  OONF_OS_SYSTEM_SUBSYSTEM,
 };
 
 static struct oonf_subsystem _oonf_stream_socket_subsystem = {
@@ -178,9 +178,11 @@ oonf_stream_add(struct oonf_stream_socket *stream_socket,
           netaddr_socket_to_string(&buf, local), strerror(errno), errno);
       goto add_stream_error;
     }
-
+    stream_socket->scheduler_entry.name = stream_socket->socket_name;
     stream_socket->scheduler_entry.process = _cb_parse_request;
 
+    snprintf(stream_socket->socket_name, sizeof(stream_socket->socket_name),
+        "tcp-server: %s", netaddr_socket_to_string(&buf, local));
     oonf_socket_add(&stream_socket->scheduler_entry);
     oonf_socket_set_read(&stream_socket->scheduler_entry, true);
   }
@@ -342,7 +344,7 @@ oonf_stream_add_managed(struct oonf_stream_managed *managed) {
     managed->config.session_timeout = 120000;
   }
 
-  managed->_if_listener.process = _cb_interface_listener;
+  managed->_if_listener.if_changed = _cb_interface_listener;
   managed->_if_listener.name = managed->_managed_config.interface;
 }
 
@@ -364,6 +366,10 @@ oonf_stream_apply_managed(struct oonf_stream_managed *managed,
 
   oonf_stream_copy_managed_config(&managed->_managed_config, config);
 
+  if (managed->config.memcookie == NULL) {
+    managed->config.memcookie = &_connection_cookie;
+  }
+
   /* set back pointers */
   managed->socket_v4.managed = managed;
   managed->socket_v6.managed = managed;
@@ -371,10 +377,10 @@ oonf_stream_apply_managed(struct oonf_stream_managed *managed,
   /* handle change in interface listener */
   if (if_changed) {
     /* interface changed, remove old listener if necessary */
-    oonf_interface_remove_listener(&managed->_if_listener);
+    os_interface_remove(&managed->_if_listener);
 
     /* create new interface listener */
-    oonf_interface_add_listener(&managed->_if_listener);
+    os_interface_add(&managed->_if_listener);
   }
 
   OONF_DEBUG(LOG_STREAM, "Apply changes for managed socket (if %s) with port %d",
@@ -384,7 +390,7 @@ oonf_stream_apply_managed(struct oonf_stream_managed *managed,
   result = _apply_managed(managed);
   if (result) {
     /* did not work, trigger interface handler to try later again */
-    oonf_interface_trigger_handler(&managed->_if_listener);
+    os_interface_trigger_handler(&managed->_if_listener);
   }
   return result;
 }
@@ -397,11 +403,11 @@ oonf_stream_apply_managed(struct oonf_stream_managed *managed,
  */
 void
 oonf_stream_remove_managed(struct oonf_stream_managed *managed, bool force) {
-  oonf_interface_remove_listener(&managed->_if_listener);
+  os_interface_remove(&managed->_if_listener);
 
   oonf_stream_remove(&managed->socket_v4, force);
   oonf_stream_remove(&managed->socket_v6, force);
-  oonf_interface_remove_listener(&managed->_if_listener);
+  os_interface_remove(&managed->_if_listener);
   oonf_stream_free_managed_config(&managed->_managed_config);
 }
 
@@ -479,19 +485,19 @@ _stream_close(struct oonf_stream_session *session) {
  */
 int
 _apply_managed(struct oonf_stream_managed *managed) {
-  struct os_interface_data *data = NULL;
+  struct os_interface *bind_socket_to_if = NULL;
 
   /* get interface */
-  if (managed->_if_listener.interface) {
-    data = &managed->_if_listener.interface->data;
+  if (!managed->_if_listener.data->flags.any) {
+    bind_socket_to_if = managed->_if_listener.data;
   }
 
-  if (_apply_managed_socket(AF_INET, managed, &managed->socket_v4, data)) {
+  if (_apply_managed_socket(AF_INET, managed, &managed->socket_v4, bind_socket_to_if)) {
     return -1;
   }
 
   if (os_system_is_ipv6_supported()) {
-    if (_apply_managed_socket(AF_INET6, managed, &managed->socket_v6, data)) {
+    if (_apply_managed_socket(AF_INET6, managed, &managed->socket_v6, bind_socket_to_if)) {
       return -1;
     }
   }
@@ -507,7 +513,7 @@ _apply_managed(struct oonf_stream_managed *managed) {
  */
 static int
 _apply_managed_socket(int af_type, struct oonf_stream_managed *managed,
-    struct oonf_stream_socket *stream, struct os_interface_data *data) {
+    struct oonf_stream_socket *stream, struct os_interface *data) {
   struct netaddr_acl *bind_ip_acl;
   const struct netaddr *bind_ip;
   union netaddr_socket sock;
@@ -516,16 +522,17 @@ _apply_managed_socket(int af_type, struct oonf_stream_managed *managed,
   bind_ip_acl = &managed->_managed_config.bindto;
 
   /* Get address the unicast socket should bind on */
-  if (data != NULL && !data->up) {
+  if (data != NULL && !data->flags.up) {
     bind_ip = NULL;
   }
-  else if (data != NULL && netaddr_get_address_family(data->linklocal_v6_ptr) == af_type &&
-      netaddr_acl_check_accept(bind_ip_acl, data->linklocal_v6_ptr)) {
+  else if (data != NULL
+      && netaddr_get_address_family(data->if_linklocal_v6) == af_type
+      && netaddr_acl_check_accept(bind_ip_acl, data->if_linklocal_v6)) {
 
-    bind_ip = data->linklocal_v6_ptr;
+    bind_ip = data->if_linklocal_v6;
   }
   else {
-    bind_ip = oonf_interface_get_bindaddress(af_type, bind_ip_acl, data);
+    bind_ip = os_interface_get_bindaddress(af_type, bind_ip_acl, data);
   }
   if (!bind_ip) {
     oonf_stream_remove(stream, true);
@@ -547,15 +554,17 @@ _apply_managed_socket(int af_type, struct oonf_stream_managed *managed,
 
     oonf_stream_remove(stream, true);
   }
-  if (oonf_stream_add(stream, &sock)) {
-    return -1;
-  }
 
   /* copy configuration */
   memcpy(&stream->config, &managed->config, sizeof(stream->config));
   if (stream->config.memcookie == NULL) {
     stream->config.memcookie = &_connection_cookie;
   }
+
+  if (oonf_stream_add(stream, &sock)) {
+    return -1;
+  }
+
   return 0;
 }
 
@@ -609,14 +618,12 @@ _create_session(struct oonf_stream_socket *stream_socket,
     struct os_fd *sock, const struct netaddr *remote_addr,
     const union netaddr_socket *remote_socket) {
   struct oonf_stream_session *session;
-#ifdef OONF_LOG_DEBUG_INFO
-  struct netaddr_str buf;
-#endif
+  struct netaddr_str nbuf1, nbuf2;
 
   /* put socket into non-blocking mode */
   if (os_fd_set_nonblocking(sock)) {
-    OONF_WARN(LOG_STREAM, "Cannot read comport socket status: %s (%d)",
-        strerror(errno), errno);
+    OONF_WARN(LOG_STREAM, "Cannot set socket %d nonblocking: %s (%d)",
+        os_fd_get_fd(sock), strerror(errno), errno);
     return NULL;
   }
 
@@ -636,16 +643,20 @@ _create_session(struct oonf_stream_socket *stream_socket,
   }
 
   os_fd_copy(&session->scheduler_entry.fd, sock);
+  session->scheduler_entry.name = session->socket_name;
   session->scheduler_entry.process = _cb_parse_connection;
-  oonf_socket_add(&session->scheduler_entry);
-  oonf_socket_set_read(&session->scheduler_entry, true);
-  oonf_socket_set_write(&session->scheduler_entry, true);
   session->send_first = stream_socket->config.send_first;
   session->stream_socket = stream_socket;
 
   session->remote_address = *remote_addr;
   session->remote_socket = *remote_socket;
 
+  /* generate socket name */
+  snprintf(session->socket_name, sizeof(session->socket_name),
+      "tcp: %s,%s",
+      netaddr_socket_to_string(&nbuf1, &stream_socket->local_socket),
+      netaddr_socket_to_string(&nbuf2, &session->remote_socket));
+
   if (stream_socket->session_counter < stream_socket->config.allowed_sessions) {
     /* create active session */
     session->state = STREAM_SESSION_ACTIVE;
@@ -663,6 +674,10 @@ _create_session(struct oonf_stream_socket *stream_socket,
     oonf_timer_start(&session->timeout, stream_socket->config.session_timeout);
   }
 
+  oonf_socket_add(&session->scheduler_entry);
+  oonf_socket_set_read(&session->scheduler_entry, true);
+  oonf_socket_set_write(&session->scheduler_entry, true);
+
   if (stream_socket->config.init) {
     if (stream_socket->config.init(session)) {
       goto parse_request_error;
@@ -670,7 +685,7 @@ _create_session(struct oonf_stream_socket *stream_socket,
   }
 
   OONF_DEBUG(LOG_STREAM, "Got connection through socket %d with %s.\n",
-      os_fd_get_fd(sock), netaddr_to_string(&buf, remote_addr));
+      os_fd_get_fd(sock), netaddr_to_string(&nbuf1, remote_addr));
 
   list_add_tail(&stream_socket->session, &session->node);
   return session;
@@ -697,10 +712,7 @@ _cb_timeout_handler(struct oonf_timer_instance *ptr) {
 
 /**
  * Handle events for TCP session from network scheduler
- * @param fd filedescriptor of TCP session
- * @param data custom data
- * @param event_read true if read-event is incoming
- * @param event_write true if write-event is incoming
+ * @param entry socket entry to be parsed
  */
 static void
 _cb_parse_connection(struct oonf_socket_entry *entry) {
@@ -778,6 +790,9 @@ _cb_parse_connection(struct oonf_socket_entry *entry) {
 
       /* still call callback once more */
       session->state = s_sock->config.receive_data(session);
+
+      /* switch off read events */
+      oonf_socket_set_read(entry, false);
     }
   }
 
@@ -866,21 +881,22 @@ _cb_parse_connection(struct oonf_socket_entry *entry) {
 
 /**
  * Callbacks for events on the interface
- * @param l
+ * @param interf os interface listener that fired
  * @return -1 if an error happened, 0 otherwise
  */
 static int
-_cb_interface_listener(struct oonf_interface_listener *l) {
+_cb_interface_listener(struct os_interface_listener *interf) {
   struct oonf_stream_managed *managed;
   int result;
 
   /* calculate managed socket for this event */
-  managed = container_of(l, struct oonf_stream_managed, _if_listener);
+  managed = container_of(interf, struct oonf_stream_managed, _if_listener);
 
   result = _apply_managed(managed);
 
   OONF_DEBUG(LOG_STREAM,
-      "Result from interface triggered socket reconfiguration: %d", result);
+      "Result from interface %s triggered socket reconfiguration: %d",
+      interf->name, result);
 
   return result;
 }
