@@ -39,6 +39,10 @@
  *
  */
 
+/**
+ * @file
+ */
+
 #ifndef OONF_VIEWER_H_
 #define OONF_VIEWER_H_
 
@@ -50,13 +54,28 @@
 #include "core/oonf_subsystem.h"
 #include "subsystems/oonf_telnet.h" /* compile-time dependency */
 
+/*! subsystem identifier */
 #define OONF_VIEWER_SUBSYSTEM "viewer"
 
+/*! viewer should return raw numbers, not iso-prefixes */
 #define OONF_VIEWER_RAW_FORMAT      "raw"
+
+/*! viewer should only return the names of the fields */
 #define OONF_VIEWER_HEAD_FORMAT     "head"
+
+/*! viewer should output json format */
 #define OONF_VIEWER_JSON_FORMAT     "json"
+
+/*! viewer should output json format with raw numbers */
 #define OONF_VIEWER_JSON_RAW_FORMAT "jsonraw"
+
+/*! viewer should output json format without surrounding object brackets */
 #define OONF_VIEWER_DATA_FORMAT     "data"
+
+/**
+ * viewer should output json format with raw numbers
+ * and without surrounding object brackets
+ */
 #define OONF_VIEWER_DATA_RAW_FORMAT "dataraw"
 
 /**
@@ -105,7 +124,7 @@ struct oonf_viewer_template {
   struct json_session _json;
 };
 
-EXPORT int oonf_viewer_output_prepare(struct oonf_viewer_template *template,
+EXPORT void oonf_viewer_output_prepare(struct oonf_viewer_template *template,
     struct abuf_template_storage *storage, struct autobuf *out, const char *format);
 EXPORT void oonf_viewer_output_print_line(struct oonf_viewer_template *template);
 EXPORT void oonf_viewer_output_finish(struct oonf_viewer_template *template);
